{******************************************************}
{                                                      }
{   Interpolate.Pas               Version 2.80         }
{   For CompuScope boards and Microsoft Windows 3.x.   }
{   Copyright (c) 1993 Gage Applied Sciences Inc.      }
{   Last Update:                  94/09/12             }
{                                                      }
{******************************************************}

unit gainter;

{$N+}
interface
Uses   Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,gage_drv,editdata;

const
     INTERPOLATE_TRIGGER_DISABLE = 32;
     boards_in_system=1;
     BLF            = 260;
     TRIG_TIMEOUT   = 10;
     BUSY_TIMEOUT   = 10;
     seg=$FFFF;
     ind=$FFFF;
     interpolate_trigger_channel= INTERPOLATE_TRIGGER_DISABLE;
     power2:array[1..15] of integer=(1,2,4,8,16,32,64,128,256,512,1024,2048,4096,8192,16384);

type
     pacqui=^tacqui;
     tacqui=array[0..32767] of byte;
type
     ptable=^ttable;
     ttable=array[0..16384] of real;

type
  srtype = record
	rate: 		word;{ Rate used for setting the CompuScope }
	mult: 		word;{ Multiplier used for setting the CompuScope }
	sr_calc: 	longint;	{ Time between samples (in ns). Used in calculateing the  }
				{ number of points.					  }
	text: 		pchar;	{ Text associated with current settings of the CompuScope.}
        mask:byte; {masque pour transfert direct}
    end;
    irtype = record
	constant:	word;{ Value of the driver constant for the input range.}
	gf_calc:	double; { Multiplier used for display voltage amplitude.   }
	gf_text:	pchar;  { Text associated with the input range.            }
    end;

    type
    pdatacard=^datacard;
    datacard=record
	srtable:    array[0..42] of srtype;
        range:     array[0..6] of irtype;
	trigger_address:    address;
	starting_address:   address;
	ending_address:	    address;
	current_address:    address;
	buffer_size:	    longint;
	buffer_mask:	    longint;
    end;

type
        pglobpar=^globpar;
        globpar=record
            typacqui:integer; {type d'acquisition en cours}
            acqvi:integer; {0:rien, 1:rafraichissement, 2:FT}
            acqrepdir:integer;
            acqrepft:integer;
            acmod:integer; {modes d'addition des spectres acquis}
            opmode: 	word;	{ Used by gage_capture_mode (). }
	    srindex:    word;
            range_a:	word;
	    couple_a:	word;
	    range_b:	word;
	    couple_b:	word;
	    source:	word;	{ source du trigger}
	    slope:	word;  {pente trigger}
	    depth:	integer;  {nombre de points}
	    level:	word;  {trigger}
	    range_e:	word;  {niveau externe}
	    couple_e:	word;  {couplage externe}
            acint:integer;
end;


var
  gage:pdatacard;
  curpa,savecurpa:pglobpar;
    board_info:             gage_driver_info_type;
     gage_board_location:    gage_b_l_array;
     interpolate_trigger:         int16;
     loc:integer;
     tab1,tab2,invtab1,invtab2:ptable;
     {interpolate_trigger_channel: int16;
      To disable the interpolated trigger use:
          interpolate_trigger_channel := INTERPOLATE_TRIGGER_DISABLE;
       otherwise use
          interpolate_trigger_channel := "low level channel number."    }
    buffer:                 array[0..BLF] of char;


      procedure init_chan_memory_pointers (board, chan, operation_mode: int16; tbs: single; var trig: int32;
                var start: int32; var endaddr: int32);
      procedure init_memory_pointers (board, operation_mode: int16; tbs: single; var trig: address;
                 var start: address; var endaddr: address);
      procedure   Timing (point1msec: word);
      function InitBoard(hwindow:hwnd): integer;
      procedure InitMemPtrs;
      procedure SetBoard(hwindow:hwnd);
      procedure AcquireStart;
      procedure tranferdat;
implementation

function InitBoard(hwindow:hwnd): integer;
var
rec:int16;
segment:uint16;
index:uint16;
typename:uint16;
status:uint16;
begin
rec:=0;
segment:=0;
index:=0;
     if gage_driver_initialize(gage_board_location, GAGE_MEMORY_SIZE_TEST) = 0 then
     begin
          MessageBox(HWindow, 'No Compuscope boards found', 'Error', mb_ok);
     end;
     {Timing (100);} { Let the relays settle. }
     gage_select_board(1);  {Select master}
end;





procedure init_interpolate_trigger (board, trigger_source, operation_mode, trigger_level, trigger_slope: int16);
begin
      gage_reset_interpolate_trigger (0, GAGE_CHAN_A, 0, 0, 0); {Disable interpolated trigger.}
end;


procedure init_chan_memory_pointers (board, chan, operation_mode: int16; tbs: single; var trig: int32;
                                    var start: int32; var endaddr: int32);
begin
    gage_select_board (board);
    gage_calculate_addresses (chan AND 1, operation_mode, tbs, trig, start, endaddr);
{    gage_get_interpolate_trigger(0,chan and 1,trig,start,endaddr);}
end;


procedure init_memory_pointers (board, operation_mode: int16; tbs: single; var trig: address;
                               var start: address; var endaddr: address);
var
    i:int16;
    trigr,startr,endaddrr:int32;
begin
     if interpolate_trigger_channel<interpolate_trigger_disable then begin
    gage_select_board (board);
    gage_calculate_addresses (interpolate_trigger_channel AND 1, operation_mode, tbs, trigr, startr, endaddrr);
{    gage_get_interpolate_trigger(0,interpolate_trigger_channel and 1,trigr,startr,endaddrr);}
     end;
     FOR i:=0 to 1 do  begin
         init_chan_memory_pointers (1, i, operation_mode, tbs, trig[i], start[i], endaddr[i]);
      end;
end;

procedure Timing (point1msec: word);
begin
     { CX: Loop Counter }
     { Pascal takes care of the stack for your. }
     asm
        mov  cx,point1msec
        jcxz @t2_fast_return   { Routine required no timeout. }

     @t2_init_timer_2:

        mov  al,10110110B      { Init T/C 2, 2 bytes, Mode 3, Binary count.}
        out  43h,al

        mov  ax,1193           { Timing constant = approximately 1,000 Hz.}
        mov  ax,119            { Timing constant = approximately 10,000 Hz.}
        out  42h,al            { Loat Timer #2. }
        mov  al,ah
        out  42h,al

        in   al,61h            { Get Current 8255 port 1 value. }
        and  al,0fch           { Clear last two bits of port. }
        or   al,1              { Turn Timer #2 GATE on. }
        out  61h,al

     @t2_wait_expire_time:

     @t2_low_to_high:

        in   al,61h            { Get timer 2 status. Use 61h because it's not an PC/XT. }
        test al,20h            { Test the Timer channel 2 bit. }
        jnz  @t2_low_to_high   { Loop until it is low. }

     @t2_wait_high:

        in   al,61h            { Get timer 2 status. }
        test al,20h            { Test the bit again. }
        jz   @t2_wait_high     { Loop until it goes high. }
        loop @t2_wait_expire_time    { Loop CX times. }

     @t2_return:
        in   al,61h            { Get current port values. }
        and  al,0fch           { Clear speaker bit. }

     @t2_fast_return:
   end;
end;

procedure InitMemPtrs;
begin
init_memory_pointers (1, curpa^.opmode, gage^.srtable[curpa^.srindex].sr_calc, gage^.trigger_address,
                          gage^.starting_address,gage^.ending_address);
end;

procedure SetBoard(hwindow:hwnd);
var
     i: int16;
     error_code: int16;
     tempstr: string[8];
     error_str: array[0..7] of char;
     j:integer;
begin
     gage_capture_mode(curpa^.opmode,gage^.srtable[curpa^.srindex].rate,gage^.srtable[curpa^.srindex].mult);
     gage_init_clock;
     gage_input_control(GAGE_CHAN_A, GAGE_INPUT_ENABLE, curpa^.couple_a, curpa^.range_a);
     if curpa^.opmode = GAGE_SINGLE_CHAN then
          gage_input_control(GAGE_CHAN_B, GAGE_INPUT_ENABLE, curpa^.couple_a, curpa^.range_a)
     else
          gage_input_control(GAGE_CHAN_B, GAGE_INPUT_ENABLE, curpa^.couple_b, curpa^.range_b);
     gage_get_driver_info(board_info);
     j:=power2[curpa^.depth];
     gage_trigger_control(curpa^.source, curpa^.couple_e, curpa^.range_e,curpa^.slope, curpa^.level,j);
      {gage_multiple_record(1);}
       gage^.buffer_size := board_info.max_available_memory;
       gage^.buffer_mask := gage^.buffer_size - 1;
end;

procedure AcquireStart;
var
    auto_trigger: int16;
begin
     if curpa^.source =GAGE_CHAN_A then
         auto_trigger := 1 else auto_trigger := 0;
     asm {gestion interuptions}
     {mov dx,0210h
     mov al,04
     out dx,al
     mov al,10100100b
     inc dx
     out dx,al}
     end;
     gage_start_capture(auto_trigger);
     end;


    procedure tranferdat;
    var
       acq:pacqui;
       blck,i:integer;
          startbloc:word;
       pdep:longint;
       mask:byte;
       ind:integer;
    begin
         pdep:=power2[curpa^.depth];
         if pdep>4096 then pdep:=4096;
         mask:=gage^.srtable[curpa^.srindex].mask;
         InitMemPtrs;
         {gage_multiple_record(0);}
         acq:=new(pacqui);
         gage_need_ram(1);
         loc := gage^.trigger_address[0];
         for i:=1 to pdep do begin
         tab1^[i]:=gage_mem_read_chan_a((i+loc) mod gage^.buffer_mask);
         ind:=und.indexof('VOIEA');
         und.spect[ind].data[i-1,0].z:=tab1^[i];
         end;
         { Do Channel B, dual channel mode only }
         loc := gage^.trigger_address[1];
         {startbloc:=gage^.trigger_address[1] div 4096;
         loc:=gage^.trigger_address[1] mod 4096;
         tfrac(pspec[2],pdep,startbloc,loc,1,mask);}
         for i:=1 to pdep do begin
         tab2^[i]:=gage_mem_read_chan_b((i+loc)mod gage^.buffer_mask);
         ind:=und.indexof('VOIEB');
         und.spect[ind].data[i-1,0].z:=tab2^[i];
         end;
         gage_need_ram(0);
         dispose(acq);
    end;

end.

