{***********************************************************}
{                                                           }
{   Start.Pas                          Version 2.85         }
{   For CompuScope boards and Microsoft Windows 3.x.        }
{   Copyright (c) 1993 Gage Applied Sciences Inc.           }
{   Last Update:                        12/02/94            }
{                                                           }
{***********************************************************}

unit Sta1012;

{$N+}
interface

uses Gage_Drv;
const
	DEFAULT_SAMPLE_RATE_INDEX  = 22;{ 10 MHz, maximum sample rate common to all CompuScope cards.}
	EXTERNAL_SAMPLE_RATE_INDEX = 42;
	RANGE_MAX				   = 7; { Maximum index modulo value for .range_a and .range_b (boarddef) items for irtype.}

        SRTI_5_MHZ	  =	0;         { Constants for use with the srtable structure. }
	SRTI_10_MHZ	  =	1;
	SRTI_25_MHZ	  =	2;
        SRTI_50_MHZ	  =	3;
	SRTI_EXTERNAL =	42;

type
  srtype = record
	rate: 		word;{ Rate used for setting the CompuScope }
	mult: 		word;{ Multiplier used for setting the CompuScope }
	sr_calc: 	longint;	{ Time between samples (in ns). Used in calculateing the  }
				{ number of points.					  }
	text: 		pchar;	{ Text associated with current settings of the CompuScope.}
        mask:byte; {masque pour transfert direct}
    end;
    irtype = record
	constant:	word;{ Value of the driver constant for the input range.}
	gf_calc:	double; { Multiplier used for display voltage amplitude.   }
	gf_text:	pchar;  { Text associated with the input range.            }
    end;

    type
    pdatacard=^datacard;
    datacard=record
	srtable:    array[0..42] of srtype;
        range:     array[0..6] of irtype;
	trigger_address:    address;
	starting_address:   address;
	ending_address:	    address;
	current_address:    address;
	buffer_size:	    longint;
	buffer_mask:	    longint;
    end;

type
        pglobpar=^globpar;
        globpar=record
            typacqui:integer; {type d'acquisition en cours}
            acqvi:integer; {0:rien, 1:rafraichissement, 2:FT}
            acqrepdir:integer;
            acqrepft:integer;
            acmod:integer; {modes d'addition des spectres acquis}
            opmode: 	word;	{ Used by gage_capture_mode (). }
	    srindex:    word;
            range_a:	word;
	    couple_a:	word;
	    range_b:	word;
	    couple_b:	word;
	    source:	word;	{ source du trigger}
	    slope:	word;  {pente trigger}
	    depth:	integer;  {nombre de points}
	    level:	word;  {trigger}
	    range_e:	word;  {niveau externe}
	    couple_e:	word;  {couplage externe}
            multiple: boolean;
            acint:integer;
end;


 var
		B_T_CS1012:				longword;
                B_T_SINGLE:				longword;
		B_T_DUAL:				longword;
		B_T_GENERIC:			longword;
  gage:pdatacard;
  curpa,savecurpa:pglobpar;
  board_info:             gage_driver_info_type;
  gage_board_location:    gage_b_l_array;
  interpolate_trigger:         int16;
  loc:integer;
  function  b_t_double_up_bits (flags: longint): longint;
  procedure init_variables;

implementation

Uses   Windows, Messages;

        function  b_t_double_up_bits (flags: longint): longint;
	{ Initialization of flags used in the sample_rate_table structure. }
	begin
		b_t_double_up_bits := ((flags SHL 16) OR flags);
	end;



	procedure init_variables;
	{ Initialization of variables used in the demonstration program }
	var
		i: integer;
	begin

			{ Board type flags for a missing CompuScope.}

		B_T_CS1012 := b_t_double_up_bits (GAGE_ASSUME_CS1012);
                B_T_SINGLE := $FFFF0000;  {Board type flags mask for single channel mode.}
		B_T_DUAL := $0000FFFF;    {Board type flags mask for dual channel mode.  }
                with gage^ do begin
		srtable[18].rate := GAGE_RATE_1;
		srtable[19].rate := GAGE_RATE_2;
                srtable[20].rate := GAGE_RATE_4;
		srtable[21].rate := GAGE_RATE_5;
                srtable[22].rate := GAGE_RATE_10;
		srtable[42].rate := GAGE_EXT_CLK;
		srtable[9].rate := GAGE_RATE_1;
		srtable[10].rate := GAGE_RATE_2;
		srtable[11].rate := GAGE_RATE_5;
		srtable[12].rate := GAGE_RATE_10;
		srtable[13].rate := GAGE_RATE_20;
		srtable[14].rate := GAGE_RATE_50;
		srtable[15].rate := GAGE_RATE_100;
		srtable[16].rate := GAGE_RATE_200;
		srtable[17].rate := GAGE_RATE_500;



                for i := 0 to 8 do
		begin
		srtable[i].mult := GAGE_MHZ;
		end;
                for i := 9 to 17 do
		begin
		srtable[i].mult := GAGE_KHZ;
		end;
                for i := 18 to 22 do srtable[i].mult := GAGE_MHZ;
{temps en nanosec}
                srtable[42].mult := GAGE_EXT_CLK;
                srtable[0].sr_calc  := 1000000000;
		srtable[1].sr_calc  :=  500000000;
		srtable[2].sr_calc  :=  200000000;
		srtable[3].sr_calc  :=  100000000;
		srtable[4].sr_calc  :=   50000000;
		srtable[5].sr_calc  :=   20000000;
		srtable[6].sr_calc  :=   10000000;
		srtable[7].sr_calc  :=    5000000;
		srtable[8].sr_calc  :=    2000000;
		srtable[9].sr_calc  :=    1000000;
		srtable[10].sr_calc :=     500000;
		srtable[11].sr_calc :=     200000;
		srtable[12].sr_calc :=     100000;
		srtable[13].sr_calc :=      50000;
		srtable[14].sr_calc :=      20000;
		srtable[15].sr_calc :=      10000; {100 KHz}
		srtable[16].sr_calc :=       5000;
		srtable[17].sr_calc :=       2000;
		srtable[18].sr_calc :=       1000; {1 MHz}
		srtable[19].sr_calc :=        500;
		srtable[20].sr_calc :=        250;
		srtable[21].sr_calc :=        200;
		srtable[22].sr_calc :=        100; {10 MHz}
		srtable[42].sr_calc :=          0;
	gage^.srtable[0].text  :=    '1 Hz';gage^.srtable[0].mask:=$34;
	gage^.srtable[1].text  :=    '2 Hz';gage^.srtable[1].mask:=$34;
	gage^.srtable[2].text  :=    '5 Hz';gage^.srtable[2].mask:=$34;
	gage^.srtable[3].text  :=   '10 Hz';gage^.srtable[3].mask:=$34;
	gage^.srtable[4].text  :=   '20 Hz';gage^.srtable[4].mask:=$34;
	gage^.srtable[5].text  :=   '50 Hz';gage^.srtable[5].mask:=$34;
	gage^.srtable[6].text  :=  '100 Hz';gage^.srtable[6].mask:=$34;
	gage^.srtable[7].text  :=  '200 Hz';gage^.srtable[7].mask:=$34;
	gage^.srtable[8].text  :=  '500 Hz';gage^.srtable[8].mask:=$34;
	gage^.srtable[9].text  :=   '1 KHz';gage^.srtable[9].mask:=$34;
	gage^.srtable[10].text :=   '2 KHz';gage^.srtable[10].mask:=$34;
	gage^.srtable[11].text :=   '5 KHz';gage^.srtable[11].mask:=$34;
	gage^.srtable[12].text :=  '10 KHz';gage^.srtable[12].mask:=$34;
	gage^.srtable[13].text :=  '20 KHz';gage^.srtable[13].mask:=$30;
	gage^.srtable[14].text :=  '50 KHz';gage^.srtable[14].mask:=$30;
	gage^.srtable[15].text := '100 KHz';gage^.srtable[15].mask:=$30;
	gage^.srtable[16].text := '200 KHz';gage^.srtable[16].mask:=$30;
	gage^.srtable[17].text := '500 KHz';gage^.srtable[17].mask:=$30;
	gage^.srtable[18].text :=   '1 MHz';gage^.srtable[18].mask:=$30;
	gage^.srtable[19].text :=   '2 MHz';gage^.srtable[19].mask:=$2C;
	gage^.srtable[20].text :=   '4 MHz';gage^.srtable[20].mask:=$28;
	gage^.srtable[21].text :=   '5 MHz';gage^.srtable[21].mask:=$24;
	gage^.srtable[22].text :=  '10 MHz';gage^.srtable[22].mask:=$20;
        gage^.srtable[42].text :=   'Ext';

        range[0].constant := GAGE_PM_10_V;
        range[0].gf_calc := 10.0;
        range[0].gf_text := '+/-10v';

        range[1].constant := GAGE_PM_5_V;
        range[1].gf_calc := 5.0;
        range[1].gf_text := '+/-5v';

        range[2].constant := GAGE_PM_2_V;
        range[2].gf_calc := 2.0;
        range[2].gf_text := '+/-2v';

        range[3].constant := GAGE_PM_1_V;
        range[3].gf_calc := 1.0;
        range[3].gf_text := '+/-1v';

        range[4].constant := GAGE_PM_500_MV;
        range[4].gf_calc := 0.5;
        range[4].gf_text := '+/-500mv';

        range[5].constant := GAGE_PM_200_MV;
        range[5].gf_calc := 0.2;
        range[5].gf_text := '+/-200mv';

        range[6].constant := GAGE_PM_100_MV;
        range[6].gf_calc := 0.1;
        range[6].gf_text := '+/-100mv';
        end;

        curpa^.opmode    := GAGE_DUAL_CHAN;
	curpa^.srindex 	:= 17;
	curpa^.range_a 	:= GAGE_PM_5_V;
	curpa^.couple_a := GAGE_AC;
	curpa^.range_b 	:= GAGE_PM_5_V;
	curpa^.couple_b := GAGE_AC;
	curpa^.source 	:= GAGE_CHAN_A;
	curpa^.slope	:= GAGE_NEGATIVE;
	curpa^.depth	:= 14;
	curpa^.level	:= 180;
	curpa^.range_e	:= GAGE_PM_5_V;
	curpa^.couple_e	:= GAGE_AC;
        curpa^.acint:=0; {horloge interne}
        curpa^.acqrepdir:=1;
        curpa^.acqrepft:=1;
        for i := 0 to 1 do
		begin
		   gage^.trigger_address[i] := 0;
		   gage^.starting_address[i]:= 0;
		   gage^.ending_address[i]  := 0;
		   gage^.current_address[i] := 0;
		end;

		gage^.buffer_size := 0;
		gage^.buffer_mask := 0;
	end;
end.

